/*
 * Copyright (c) 2020 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "bands.h"
#include "qso.h"
#include "xlua_impl.h"

/*
 * Call a function with the qso as an argument.  If ro==true, struct qso
 * cannot be modified by the lua script.
 *
 * Returns:
 *  -ENOENT: function not found
 */
int qso_lua_call(void *state, const char *funcname, struct qso *qso, bool ro,
		 char *error, size_t errlen)
{
	int ret;

	ret = lua_getglobal(state, funcname);
	if (ret != LUA_TFUNCTION) {
		lua_pop(state, 2);
		CHECK_STACK(state);

		strcpy_safe(error, "not a function", errlen);

		return -ENOENT;
	}

	xlua_pushqso(state, qso, ro);

	/*
	 * now invoke the function
	 */
	ret = lua_pcall(state, 1, 1, 0);
	if (ret)
		COPY_ERROR(state, -1, error, errlen);

	lua_pop(state, 1);
	CHECK_STACK(state);

	return lua2errno(ret);
}
